﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading.Tasks.Dataflow;
using Models.Events;
using Models.Rules;

namespace Models.Game
{
    public class Game
    {
        private bool _isRunning;
        public Player CurrentPlayer { get; private set; }

        public Map UsedMap { get; private set; }

        private Dice Dice1 { get; }
        private Dice Dice2 { get; }

        private int Turn { get; set; }

        public Rules.Rules GameRules { get; }

        // == Events ==
        public event EventHandler<GameStartedEventArgs> GameStarted;
        public event EventHandler<GameEndedEventArgs> GameEnded;
     
        
        public Game(Player player, Map map)
        {
            _isRunning = false;
            UsedMap = map;
            CurrentPlayer = player;
            Dice1 = new Dice();
            Dice2 = new Dice(1);
            Turn = 1;
            GameRules = new Rules.Rules();
        }

        public void RollAllDice()
        {
            Dice1.Roll();
            Dice2.Roll();
        }

        /// <summary>
        /// Performs an operation on the values of two dice based on the provided operation.
        /// </summary>
        /// <param name="o">The operation to perform. This can be LOWER, HIGHER, SUBTRACTION, ADDITION, or MULTIPLICATION.</param>
        /// <returns>
        /// The result of the operation. If the operation is LOWER or HIGHER, it returns the lower or higher value of the two dice respectively.
        /// If the operation is SUBTRACTION, it returns the difference between the higher and lower value of the two dice.
        /// If the operation is ADDITION, it returns the sum of the values of the two dice.
        /// If the operation is MULTIPLICATION, it returns the product of the values of the two dice.
        /// If the operation is not one of the operations, it throws an ArgumentOutOfRangeException.
        /// </returns>
        public int ResultOperation(Operation o)
        {
            switch (o)
            {
                case Operation.LOWER:
                    return Dice1.IsLower(Dice2) ? Dice1.Value : Dice2.Value;

                case Operation.HIGHER:
                    return Dice1.IsLower(Dice2) ? Dice2.Value : Dice1.Value;

                case Operation.SUBTRACTION:
                    return Dice1.IsLower(Dice2) ? Dice2.Value - Dice1.Value : Dice1.Value - Dice2.Value;

                case Operation.ADDITION:
                    return Dice2.Value + Dice1.Value;

                case Operation.MULTIPLICATION:
                    return Dice2.Value * Dice1.Value;
                
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

        public void PlaceResult(Cell playerChoice, int result)
        {
            if (Turn == 1 || GameRules.NearCellIsValid(playerChoice, UsedMap.Boards))
            {
                playerChoice.Value = result;
            }
        }

        public void AddToRopePath(Cell playerChoice, List<List<Cell>> ropePaths,Cell adjacente)
        {

            List<List<Cell>> possiblePaths;

            // {adjacentes} ne doit pas etre egale a {playerChoice}
            if (adjacente.Value == playerChoice.Value) return;

            // {adjacentes} doit etre croissant ou decroissant
            if ((adjacente.Value - playerChoice.Value > 1) || (adjacente.Value - playerChoice.Value < -1)) return;

            // Un nombre ne peut appartenir qu'a un seul chemin de corde
            foreach (List<Cell> paths in ropePaths)
            {
                foreach (var cells in paths)
                {
                    if (adjacente.X == cells.X && adjacente.Y == adjacente.Y)
                    {
                        possiblePaths.Add(paths);
                    }


                }

            }


            // Si {ropePaths} est vide, il faut alors creer un chemin de corde entre {playerChoice}
            if (ropePaths.Count == 0)
            {

            }
        }
        
        /// <summary>
        /// Initializes the game.
        /// </summary>
        public void InitializeGame()
        {
            _isRunning = true;
            GameStarted?.Invoke(this, new GameStartedEventArgs(CurrentPlayer));
            GameLoop();
        }
        

        private async void GameLoop()
        {
            while (_isRunning)
            {
                
                if (CheckGameEnd()) //TODO Règle pour check si fin de jeu
                {
                    _isRunning = false;
                    //TODO Code pour comptabiliser les points
                    GameEnded?.Invoke(this, new GameEndedEventArgs(//player));
                }

                await Task.Delay(1000); // 1 seconde
            }
        }

    }
}
